SystemOrganization addCategory: #'XPF-CodeMarker'!
SystemOrganization addCategory: #'XPF-Tests-Models'!
SystemOrganization addCategory: #'XPF-Models'!
SystemOrganization addCategory: #'XPF-Seaside-Components'!
SystemOrganization addCategory: #'XPF-Seaside-ComponentsTest'!
SystemOrganization addCategory: #'XPF-CodeMarker-Tests'!
SystemOrganization addCategory: #'XPF-CodeMarker-Commands'!
SystemOrganization addCategory: #'XPF-Rest'!

!String methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/20/2009 17:17'!
findAllStrings: aString
	"Returns a collection with all starting indices of the given string in me."
	
	|indices currentStartIndex|
	indices := Bag new.
	currentStartIndex := 1.
	
	[currentStartIndex := (self findString: aString startingAt: currentStartIndex) + 1.
		currentStartIndex > 1 
	] whileTrue: [indices add: currentStartIndex - 1].

	^indices.! !

!OBCmdMultiView methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/3/2009 15:53'!
execute
	|viewType|
	viewType := OBChoiceRequest labels: #(Source Bytecode Decompile PrettyPrint XPF).
	self styler: SHTextStylerST80 new.
	viewType ifNotNil: [self perform: (#toggle, viewType) asSymbol]! !

!OBCmdMultiView methodsFor: '*XPF-CodeMarker' stamp: 'cs 1/15/2009 17:24'!
styler: styler
	| panel morph |
	
	panel := requestor browser panels 
		detect: [ :each | each isKindOf: OBDefinitionPanel ]
		ifNone: [ ^ self ].
	morph := OBPluggableTextMorph allSubInstances
		detect: [ :each | each model == panel ]
		ifNone: [ ^ self ].
	morph styler: (styler
		view: morph;
		yourself).! !

!OBCmdMultiView methodsFor: '*XPF-CodeMarker' stamp: 'cs 1/15/2009 17:22'!
toggleXPF

	self styler: XPFTextStyler new.
	self setTextInDefinitionPanel: target source.! !

HttpService subclass: #XPFHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Rest'!

XPFHttpService class
	instanceVariableNames: 'service port'!

XPFHttpService class
	instanceVariableNames: 'service port'!

!XPFHttpService class methodsFor: 'as yet unclassified' stamp: 'mt 1/30/2009 13:34'!
createService
	^ XPFHttpService on: port.! !

!XPFHttpService class methodsFor: 'as yet unclassified' stamp: 'mt 2/2/2009 10:26'!
run
	self stop.
	
	port := 8080.
	service := self createService.
	service
		installHandler: XPFMetaHandler on: '/meta';
		installHandler: XPFDefaultHandler on: '/';
		start.! !

!XPFHttpService class methodsFor: 'as yet unclassified' stamp: 'mt 1/30/2009 13:28'!
stop
	service ifNotNil:
		[service unregister.
		service := nil].
	port := nil.! !

!XPFHttpService methodsFor: 'as yet unclassified' stamp: 'mt 2/2/2009 10:12'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!XPFHttpService methodsFor: 'as yet unclassified' stamp: 'mt 2/2/2009 10:12'!
installHandler: aRequestHandlerClass on: aPathString 
	handlers add: aPathString -> aRequestHandlerClass! !

!XPFHttpService methodsFor: 'as yet unclassified' stamp: 'mt 2/2/2009 10:27'!
processHttpRequest: request
	
	| activeHandler path |
	activeHandler := (handlers detect: [:ass| 
		path := ass key.
		request url includesSubString: path
		]) value.
	^(activeHandler new) process: request! !

TestCase subclass: #XPFCodeMarkCommandTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Tests'!

!XPFCodeMarkCommandTest methodsFor: 'as yet unclassified' stamp: 'cs 1/27/2009 17:27'!
testCodeMarkerInContextMenu
	| dialog |
	dialog :=
		(ChooseDropListDialogWindow new)
		title: 'When you yellow-button click into a Code Pane, is there a "Mark Code" item?';
		list: (OrderedCollection  with: 'Yes' with: 'No');
		openInHand.
		
	self should: dialog selectedItem = 'Yes'.! !

TestCase subclass: #XPFTestCase
	instanceVariableNames: 'originalDatabase'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

XPFTestCase subclass: #XPFDatabaseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFDatabaseTest methodsFor: 'tests' stamp: 'mt 2/12/2009 12:04'!
testAddRemoveEntity
	| entity |
	entity := XPFEntity new.
	
	XPFEntity database register: entity.
	self should: ((XPFEntity database allOf: XPFEntity) includes: entity).
	
	XPFEntity database unregister: entity.
	self shouldnt: ((XPFEntity database allOf: XPFEntity) includes: entity).
	! !

!XPFDatabaseTest methodsFor: 'tests' stamp: 'mt 2/12/2009 12:05'!
testLinkEntities
	| entity1 entity2 |
	entity1 := XPFEntity new.
	entity2 := XPFEntity new.
	
	XPFEntity database register: entity1; register: entity2.
	
	XPFEntity database link: entity1 and: entity2 named: #testlink.
	
	self should: 
		((XPFEntity database targetsFor: entity1 linkNamed: #testlink class: XPFEntity) includes: entity2).

	XPFEntity database unregister: entity1; unregister: entity2.
! !

XPFTestCase subclass: #XPFEntityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFEntityTest methodsFor: 'as yet unclassified' stamp: 'topa 2/9/2009 16:42'!
testRegistration

	| dbCount entity |
	dbCount  := (XPFEntity database allOf: XPFEntity) size.
	
	entity := XPFEntity new.
	entity register.
	self should:  (XPFEntity database allOf: XPFEntity) size = (dbCount + 1).
	
	entity unregister.
	self should:  (XPFEntity database allOf: XPFEntity) size = dbCount.! !

XPFTestCase subclass: #XPFTaskTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFTaskTest methodsFor: 'as yet unclassified' stamp: 'topa 2/9/2009 22:23'!
testNewAndDelete
	| task oldCount |
	
	oldCount := XPFTask all size.
	
	task := XPFTask newAndRegister: [ :newTask | 
		newTask
			taskName: 'test task';
			taskDescription: 'this is a test task' ;
			yourself].
		
	self should: XPFTask all size = (oldCount + 1).
	self should: (XPFTask all anySatisfy: [ :each | each taskName = 'test task' ]).
	
	task unregister.
	
	self should: XPFTask all size = oldCount.! !

!XPFTestCase class methodsFor: 'testing' stamp: 'SR 1/22/2009 11:46'!
isAbstract
 	^self name = #XPFTestCase! !

!XPFTestCase methodsFor: 'testing' stamp: 'SR 1/22/2009 11:34'!
checkAccessor: aSelector on: aTargetObject with: anObject
	"test getter/setter aSelector of aTargetObject with anObject"
	
	aTargetObject perform: aSelector asMutator with: anObject.
	self
		assert: (aTargetObject perform: aSelector) == anObject
		! !

!XPFTestCase methodsFor: 'running' stamp: 'mt 2/12/2009 12:01'!
setUp

	originalDatabase := XPFEntity database.
	XPFEntity database: XPFTestDatabase new.
! !

!XPFTestCase methodsFor: 'running' stamp: 'mt 2/12/2009 12:06'!
tearDown
	
	XPFEntity database: originalDatabase.
! !

XPFTestCase subclass: #XPFUserStoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFUserStoryTest methodsFor: 'tests' stamp: 'topa 2/10/2009 16:30'!
testAttributes
	| us |
	us := XPFUserStory new.
	self
		checkAccessor: #storyName on: us with: 'US + Tasks verwalten';
		checkAccessor: #storyDescription on: us with: 'Dinge tun und Sachen machen';
		checkAccessor: #storyPriority on: us with: 5;
		checkAccessor: #storyEstimate on: us  with: 10;
		checkAccessor: #storyActual on: us  with: 12;
		checkAccessor: #storyTasks on: us  with: (Set with: XPFTask new with: XPFTask new). 
		
! !

!XPFUserStoryTest methodsFor: 'tests' stamp: 'topa 2/10/2009 17:06'!
testHasTasks
	
	| userStory taskCount task|
	taskCount := XPFTask all size.
	userStory := XPFUserStory newAndRegister: [ :newUserStory | newUserStory ].
	self should: (userStory storyTasks size = 0).
	task := userStory storyTasks add: (XPFTask newAndRegister: [ :newTask | newTask ]).
	self should: (userStory storyTasks size = 1).
	self should: (XPFTask all size = taskCount + 1).
	userStory storyTasks remove: task ifAbsent: [self fail].
	self should: (userStory storyTasks size = 0).
	self should: (XPFTask all size = taskCount).
	! !

!XPFUserStoryTest methodsFor: 'tests' stamp: 'topa 2/9/2009 22:23'!
testNewAndDelete
	| userStory oldCount |
	
	oldCount := XPFUserStory all size.
	
	userStory := XPFUserStory newAndRegister: [ :newUserStory | 
		newUserStory
			storyName: 'test story';
			storyDescription: 'this is a test story';
			storyPriority: 3;
			storyEstimate: 10 ].
		
	self should: XPFUserStory all size = (oldCount + 1).
	self should: (XPFUserStory all anySatisfy: [ :each | each storyName = 'test story' ]).
	
	userStory unregister.
	
	self should: XPFUserStory all size = oldCount.
	! !

XPFTestCase subclass: #XPFUserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFUserTest methodsFor: 'as yet unclassified' stamp: 'mt 2/10/2009 15:33'!
testAttributes
	| user |
	user := XPFUser new.
	self
		checkAccessor: #userLogin on: user with: 'marcel.taeumel';
		checkAccessor: #userFirstName on: user with: 'Marcel';
		checkAccessor: #userLastName on: user with: 'Taeumel';
		checkAccessor: #userEMail on: user with: 'marcel.taeumel@student.hpi.uni-potsdam.de'.! !

!XPFUserTest methodsFor: 'as yet unclassified' stamp: 'mt 2/10/2009 15:46'!
testNewAndDelete
	| user oldCount |
	
	self shouldnt: (XPFUser all anySatisfy: [ :each | each userLogin = 'stefanc.richter' ]).
	
	oldCount := XPFUser all size.
	
	user := XPFUser newAndRegister: [ :newUser | 
		newUser
			userLogin: 'stefanc.richter';
			userFirstName: 'Stefan';
			userLastName: 'Richter';
			userEMail: 'stefanc.richter@student.hpi.uni-potsdam.de'].
		
	self should: XPFUser all size = (oldCount + 1).
	self should: (XPFUser all anySatisfy: [ :each | each userLogin = 'stefanc.richter' ]).
	
	user unregister.
	
	self should: XPFUser all size = oldCount.! !

!Text methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 17:42'!
removeAttributeOfKind: attClass from: start to: stop 
	"Removes an attribute kind over the interval start to stop."
	runs :=  runs copyReplaceFrom: start to: stop
			with: ((runs copyFrom: start to: stop)
				mapValues:
				[:attributes | attributes reject: [:each | each isKindOf: attClass]])! !

OBTextMorphEditorWithShout subclass: #XPFEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker'!

!XPFEditor class methodsFor: 'as yet unclassified' stamp: 'mt 2/5/2009 17:36'!
initializeCmdKeyShortcuts
	super initializeCmdKeyShortcuts.
	CmdActions
		at: ($q asInteger + 1) put: #markAsGood:;
		at: ($r asInteger + 1) put: #markAsBad:;
		at: ($t asInteger + 1) put: #markAsUnclear:;
		at: ($w asInteger + 1) put: #removeMarkup:.
	
! !

!XPFEditor methodsFor: 'as yet unclassified' stamp: 'mt 1/23/2009 14:43'!
markAsBad: aCharacterStream
	paragraph setBackgroundForSelectionWith: Color red.
	self userHasEdited.
	^ true

	
! !

!XPFEditor methodsFor: 'as yet unclassified' stamp: 'mt 1/23/2009 14:44'!
markAsGood: aCharacterStream
	paragraph setBackgroundForSelectionWith: Color green.
	self userHasEdited.
	^ true
	
! !

!XPFEditor methodsFor: 'as yet unclassified' stamp: 'mfp 2/3/2009 16:17'!
markAsUnclear: aCharacterStream
	paragraph setBackgroundForSelectionWith: Color lightGray.
	self userHasEdited.
	^ true
	
! !

!XPFEditor methodsFor: 'as yet unclassified' stamp: 'mt 2/5/2009 17:36'!
removeMarkup: aCharacterStream
	paragraph removeBackgroundForSelection.
	self userHasEdited.
	^ true

	
! !

SCComponentTestCase subclass: #XPFComponentTestCase
	instanceVariableNames: 'originalDatabase'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-ComponentsTest'!

!XPFComponentTestCase class methodsFor: 'testing' stamp: 'topa 2/10/2009 16:08'!
isAbstract
 	^self name = #XPFComponentTestCase! !

!XPFComponentTestCase methodsFor: 'as yet unclassified' stamp: 'topa 2/10/2009 16:07'!
setUp

	originalDatabase := XPFEntity database.
	XPFEntity database: XPFTestDatabase instance reset.
! !

!XPFComponentTestCase methodsFor: 'as yet unclassified' stamp: 'topa 2/10/2009 16:07'!
tearDown
	
	XPFTestDatabase instance reset.
	XPFEntity database: originalDatabase.
! !

XPFComponentTestCase subclass: #XPFTaskComponentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-ComponentsTest'!

!XPFTaskComponentTest methodsFor: 'running' stamp: 'topa 2/10/2009 16:11'!
setUp
	super setUp.
	self 
		newApplicationWithRootClass: XPFTaskComponent;
		establishSession.! !

!XPFTaskComponentTest methodsFor: 'tests' stamp: 'topa 1/23/2009 15:50'!
testNewTask
	self
		followAnchor: (self lastResponse anchorWithLabel: 'Add Task');
		assert: (self lastResponse forms size = 1).
	self testTaskForm: self lastResponse forms first.
		! !

!XPFTaskComponentTest methodsFor: 'testing' stamp: 'topa 2/10/2009 15:30'!
testTaskForm: aForm
	self 
		assert: aForm textInputs size = 2 description: 'A task should have 2 data inputs';
		assert: aForm buttons size = 3 description: 'A task should be addable, yet the addition canceable';
		assert: aForm buttons second value = 'Save';
		assert: aForm buttons third value = 'Cancel'.! !

XPFComponentTestCase subclass: #XPFUserStoryComponentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-ComponentsTest'!

!XPFUserStoryComponentTest methodsFor: 'running' stamp: 'topa 2/10/2009 16:11'!
setUp
	super setUp.
	self 
		newApplicationWithRootClass: XPFUserStoryComponent;
		establishSession.! !

!XPFUserStoryComponentTest methodsFor: 'tests' stamp: 'topa 1/23/2009 16:17'!
testNewUserStory
	self
		followAnchor: (self lastResponse anchorWithLabel: 'Add User Story');
		assert: (self lastResponse forms size = 1).
	self testUserStoryForm: self lastResponse forms first.
		! !

!XPFUserStoryComponentTest methodsFor: 'testing' stamp: 'topa 1/23/2009 16:17'!
testUserStoryForm: aForm
	self 
		assert: aForm textInputs size = 5 description: 'A task should have 5 data inputs';
		assert: aForm buttons size = 3 description: 'A user story should be addable, yet the addition canceable';
		assert: aForm buttons first value = 'Default'; "this seems to be Magritte specific"
		assert: aForm buttons second value = 'Save';
		assert: aForm buttons third value = 'Cancel'.! !

WAComponent subclass: #XPFRootComponent
	instanceVariableNames: 'userStoryAdmin taskAdmin navigation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-Components'!

!XPFRootComponent class methodsFor: 'as yet unclassified' stamp: 'mig 1/23/2009 13:58'!
canBeRoot
	^ true! !

!XPFRootComponent methodsFor: 'rendering' stamp: 'topa 1/23/2009 15:10'!
children
	^ Array with: self navigation! !

!XPFRootComponent methodsFor: 'initialize-release' stamp: 'topa 1/23/2009 15:09'!
initialize
	super initialize.
	self
		userStoryAdmin: XPFUserStoryComponent new;
		taskAdmin: XPFTaskComponent new;
		initializeNavigation! !

!XPFRootComponent methodsFor: 'initialize-release' stamp: 'topa 1/23/2009 15:12'!
initializeNavigation
	self navigation: WASimpleNavigation new.
	(self navigation)
		add: self userStoryAdmin label: 'User Stories';
		add: self taskAdmin label: 'Tasks';
		select: self navigation options first! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'topa 1/23/2009 15:10'!
navigation
	^ navigation! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'topa 1/23/2009 15:12'!
navigation: anObject 
	navigation := anObject! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:16'!
taskAdmin
	^ taskAdmin! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:16'!
taskAdmin: anObject
	taskAdmin := anObject! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:04'!
userStoryAdmin
	^ userStoryAdmin! !

!XPFRootComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:04'!
userStoryAdmin: anObject
	userStoryAdmin := anObject! !

WAComponent subclass: #XPFTaskComponent
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-Components'!

!XPFTaskComponent class methodsFor: 'as yet unclassified' stamp: 'mig 1/23/2009 14:06'!
canBeRoot
	^ false! !

!XPFTaskComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:14'!
children
	^ Array with: report! !

!XPFTaskComponent methodsFor: 'actions' stamp: 'topa 2/9/2009 22:20'!
createNewTask

	XPFTask newAndRegister: [:newTask |
		self call: newTask asComponent addValidatedForm ].
	self refreshReport.! !

!XPFTaskComponent methodsFor: 'actions' stamp: 'topa 2/9/2009 22:22'!
deleteTask: aTask

	(self confirm: ('Do you really want to delete the task named "{1}"?' format: {aTask taskName}))
		ifTrue: [aTask unregister. self refreshReport.].! !

!XPFTaskComponent methodsFor: 'actions' stamp: 'mig 1/23/2009 14:13'!
editTask: aTask
	(self call: (aTask description asComponentOn: aTask) addValidatedForm)
		ifNotNil: [self refreshReport].! !

!XPFTaskComponent methodsFor: 'initialize-release' stamp: 'mig 1/23/2009 14:07'!
initialize
	super initialize.
	self initializeReport.! !

!XPFTaskComponent methodsFor: 'initialize-release' stamp: 'mig 1/23/2009 14:09'!
initializeReport
	self report: (MAReport
 		rows: XPFTask all
 		description: XPFTask new descriptionReport).
		
	self report
		addColumn: (MACommandColumn new
			addCommandOn: self selector: #editTask: text: 'edit';
			yourself);
		addColumn: (MACommandColumn new
			addCommandOn: self selector: #deleteTask: text: 'delete';
			yourself).! !

!XPFTaskComponent methodsFor: 'updating' stamp: 'mig 1/23/2009 14:13'!
refreshReport
	self report rows: XPFTask all.! !

!XPFTaskComponent methodsFor: 'rendering' stamp: 'mig 1/23/2009 14:41'!
renderContentOn: html
	html div with: [
		html div 
			with: self report.
		html anchor 
			callback: [self createNewTask]; 
			with: 'Add Task'. ]! !

!XPFTaskComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:06'!
report
	^ report! !

!XPFTaskComponent methodsFor: 'accessing' stamp: 'mig 1/23/2009 14:06'!
report: anObject
	report := anObject! !

WAComponent subclass: #XPFUserStoryComponent
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Seaside-Components'!

!XPFUserStoryComponent class methodsFor: 'as yet unclassified' stamp: 'mig 1/23/2009 13:58'!
canBeRoot
	^false! !

!XPFUserStoryComponent methodsFor: 'accessing' stamp: 'SR 1/22/2009 14:44'!
children
	^Array with: self report.! !

!XPFUserStoryComponent methodsFor: 'as yet unclassified' stamp: 'topa 2/9/2009 22:20'!
createNewUserStory

	XPFUserStory newAndRegister: [:newUserStory |
		self call: newUserStory asComponent addValidatedForm ].
	self refreshReport.
! !

!XPFUserStoryComponent methodsFor: 'as yet unclassified' stamp: 'topa 2/9/2009 22:23'!
deleteUserStory: aUserStory

	(self confirm: ('Do you really want to delete the user story named "{1}"?' format: {aUserStory storyName})) ifTrue: [aUserStory unregister. self refreshReport ]! !

!XPFUserStoryComponent methodsFor: 'as yet unclassified' stamp: 'mig 1/23/2009 12:32'!
editUserStory: aUserStory
	(self call: (aUserStory description asComponentOn: aUserStory) addValidatedForm)
		ifNotNil: [self refreshReport].! !

!XPFUserStoryComponent methodsFor: 'initialization' stamp: 'SR 1/22/2009 14:33'!
initialize
	super initialize.
	self initializeReport.! !

!XPFUserStoryComponent methodsFor: 'initialization' stamp: 'mig 1/23/2009 12:32'!
initializeReport
	self report: (MAReport
 		rows: XPFUserStory all
 		description: XPFUserStory new descriptionReport).
		
	self report
		addColumn: (MACommandColumn new
			addCommandOn: self selector: #editUserStory: text: 'edit';
			yourself);
		addColumn: (MACommandColumn new
			addCommandOn: self selector: #deleteUserStory: text: 'delete';
			yourself).! !

!XPFUserStoryComponent methodsFor: 'as yet unclassified' stamp: 'mig 1/23/2009 12:32'!
refreshReport
	self report rows: (XPFUserStory all).! !

!XPFUserStoryComponent methodsFor: 'rendering' stamp: 'mig 1/23/2009 14:41'!
renderContentOn: html
	html div with: [
		html div 
			with: self report.
		html anchor 
			callback: [self createNewUserStory]; 
			with: 'Add User Story'. ]! !

!XPFUserStoryComponent methodsFor: 'accessing' stamp: 'SR 1/22/2009 14:45'!
report
	^ report! !

!XPFUserStoryComponent methodsFor: 'accessing' stamp: 'SR 1/22/2009 14:45'!
report: anObject
	report := anObject! !

Object subclass: #XPFDatabase
	instanceVariableNames: 'entities links'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

XPFDatabase class
	instanceVariableNames: 'instance'!

XPFDatabase class
	instanceVariableNames: 'instance'!

!XPFDatabase methodsFor: 'accessing' stamp: 'mig 1/29/2009 19:25'!
allOf: anEntityClass
	^ entities select: [:entity | entity isKindOf: anEntityClass ]! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/9/2009 16:19'!
entities
	^entities! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/9/2009 16:19'!
entities: aCollection
	entities := aCollection.! !

!XPFDatabase methodsFor: 'initialization' stamp: 'topa 2/9/2009 16:44'!
initialize
	self
		links: Set new;
		entities: Set new.! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/9/2009 16:19'!
link: anEntity and: anotherEntity named: aLinkname 
	"creates an undirected link between the two entities"
	
	self links add: (XPFLink newFor: {  anEntity. anotherEntity} asSet named: aLinkname).! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/9/2009 16:18'!
links
	^links! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/10/2009 15:27'!
links: aCollection
	links := aCollection.! !

!XPFDatabase methodsFor: 'registration' stamp: 'mig 1/29/2009 19:21'!
register: anEntity
	"registers an entity to the database"
	
	entities add: anEntity! !

!XPFDatabase methodsFor: 'accessing' stamp: 'topa 2/9/2009 16:43'!
targetsFor: anEntity linkNamed: aLinkname class: aClass 
	^ ((self links
		select: [:link | (link endpoints includes: anEntity) and: [link linkname = aLinkname]])
		inject: Set new into: [:all :link | all union: link endpoints])
		select: [:endpoint | (endpoint ~= anEntity) and: [endpoint isKindOf: aClass]]! !

!XPFDatabase methodsFor: 'registration' stamp: 'mig 1/29/2009 19:21'!
unregister: anEntity
	"removes an entity from the database"
	
	entities remove: anEntity! !

XPFDatabase subclass: #XPFTestDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Tests-Models'!

!XPFTestDatabase methodsFor: 'initialization' stamp: 'mt 2/12/2009 12:00'!
initialize
	super initialize.
	"Implement fixture setting here."! !

Object subclass: #XPFEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

XPFEntity class
	instanceVariableNames: 'database'!

XPFEntity class
	instanceVariableNames: 'database'!

!XPFEntity class methodsFor: 'accessing' stamp: 'topa 2/10/2009 16:09'!
all
	^ self database allOf: self! !

!XPFEntity class methodsFor: 'accessing' stamp: 'topa 2/10/2009 16:04'!
baseDatabase
	^ database! !

!XPFEntity class methodsFor: 'accessing' stamp: 'topa 2/10/2009 16:05'!
baseDatabase: anXPFDatabase
	database := anXPFDatabase.! !

!XPFEntity class methodsFor: 'accessing' stamp: 'topa 2/10/2009 16:04'!
database
	^ XPFEntity baseDatabase! !

!XPFEntity class methodsFor: 'accessing' stamp: 'topa 2/10/2009 16:04'!
database: anXPFDatabase
	XPFEntity baseDatabase: anXPFDatabase.! !

!XPFEntity class methodsFor: 'as yet unclassified' stamp: 'mt 2/12/2009 12:01'!
initialize
	super initialize.
	self database: XPFDatabase new.! !

!XPFEntity class methodsFor: 'accessing' stamp: 'test 2/10/2009 15:25'!
newAndRegister: aBlockContext
	| entity |
	entity := aBlockContext value: self new.
	entity ifNotNil: [entity register].
	^ entity! !

!XPFEntity methodsFor: 'registry' stamp: 'topa 2/9/2009 16:04'!
register
	"register with the database"
	self class database register: self.! !

!XPFEntity methodsFor: 'registry' stamp: 'topa 2/9/2009 16:04'!
unregister
	"unregister from database"
	self class database unregister: self.! !

XPFEntity subclass: #XPFTask
	instanceVariableNames: 'taskName taskDescription'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

!XPFTask class methodsFor: 'description' stamp: 'mig 1/23/2009 13:54'!
descriptionDescription
	^MAMemoDescription new
		selectorAccessor: #taskDescription;
		label: 'Description';
		priority: 20;
		yourself! !

!XPFTask class methodsFor: 'description' stamp: 'mig 1/23/2009 13:54'!
descriptionName
	^MAStringDescription new
		selectorAccessor: #taskName;
		label: 'Name';
		priority: 10;
		default: 'New Task';
		yourself! !

!XPFTask methodsFor: 'accessing' stamp: 'mig 1/23/2009 13:49'!
descriptionReport
	^super description 
		select: [:each | #(taskName taskDescription) includes: each accessor selector]! !

!XPFTask methodsFor: 'accessing' stamp: 'mig 1/23/2009 13:48'!
taskDescription
	^ taskDescription! !

!XPFTask methodsFor: 'accessing' stamp: 'mig 1/23/2009 13:48'!
taskDescription: anObject
	taskDescription := anObject! !

!XPFTask methodsFor: 'accessing' stamp: 'mig 1/23/2009 13:48'!
taskName
	^ taskName! !

!XPFTask methodsFor: 'accessing' stamp: 'mig 1/23/2009 13:48'!
taskName: anObject
	taskName := anObject! !

XPFEntity subclass: #XPFUser
	instanceVariableNames: 'userLogin userFirstName userLastName userEMail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

!XPFUser class methodsFor: 'description' stamp: 'mt 2/10/2009 16:07'!
descriptionEMail
	^MAStringDescription new
		selectorAccessor: #userEMail;
		label: 'E-Mail';
		priority: 30;
		default: '';
		yourself! !

!XPFUser class methodsFor: 'description' stamp: 'mt 2/10/2009 16:06'!
descriptionFirstName
	^MAStringDescription new
		selectorAccessor: #userFirstName;
		label: 'First name';
		priority: 20;
		default: '';
		yourself! !

!XPFUser class methodsFor: 'description' stamp: 'mt 2/10/2009 16:07'!
descriptionLastName
	^MAStringDescription new
		selectorAccessor: #userLastName;
		label: 'Last name';
		priority: 20;
		default: '';
		yourself! !

!XPFUser class methodsFor: 'description' stamp: 'mt 2/10/2009 16:05'!
descriptionLogin
	^MAStringDescription new
		selectorAccessor: #userLogin;
		label: 'Login';
		priority: 10;
		default: '';
		yourself! !

!XPFUser methodsFor: 'description' stamp: 'mt 2/10/2009 16:09'!
descriptionReport
	^super description 
		select: [:each | #(userLogin userFirstName userLastName userEMail) includes: each accessor selector]! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userEMail
	^ userEMail! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userEMail: anObject
	userEMail := anObject! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userFirstName
	^ userFirstName! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userFirstName: anObject
	userFirstName := anObject! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userLastName
	^ userLastName! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userLastName: anObject
	userLastName := anObject! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userLogin
	^ userLogin! !

!XPFUser methodsFor: 'accessing' stamp: 'mt 2/10/2009 15:44'!
userLogin: anObject
	userLogin := anObject! !

XPFEntity subclass: #XPFUserStory
	instanceVariableNames: 'storyName storyDescription storyPriority storyEstimate storyActual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

!XPFUserStory class methodsFor: 'description' stamp: 'SR 1/22/2009 15:08'!
descriptionActual
	^MANumberDescription new
		selectorAccessor: #storyActual;
		label: 'Actual';
		priority: 40;
		default: 0;
		yourself! !

!XPFUserStory class methodsFor: 'description' stamp: 'SR 1/22/2009 15:08'!
descriptionDescription
	^MAMemoDescription new
		selectorAccessor: #storyDescription;
		label: 'Description';
		priority: 20;
		yourself! !

!XPFUserStory class methodsFor: 'description' stamp: 'SR 1/22/2009 15:08'!
descriptionEstimate
	^MANumberDescription new
		selectorAccessor: #storyEstimate;
		label: 'Estimate';
		priority: 40;
		default: 0;
		yourself! !

!XPFUserStory class methodsFor: 'description' stamp: 'SR 1/22/2009 15:08'!
descriptionName
	^MAStringDescription new
		selectorAccessor: #storyName;
		label: 'Name';
		priority: 10;
		default: 'New User Story';
		yourself! !

!XPFUserStory class methodsFor: 'manipulation' stamp: 'SR 1/22/2009 15:07'!
descriptionPriority
	^MANumberDescription new
		selectorAccessor: #storyPriority;
		label: 'Priority';
		priority: 30;
		default: 3;
		yourself! !

!XPFUserStory class methodsFor: 'description' stamp: 'topa 2/9/2009 15:40'!
descriptionTasks
	! !

!XPFUserStory methodsFor: 'description' stamp: 'cs 1/26/2009 14:44'!
descriptionReport
	^super description 
		select: [:each | #(storyName storyEstimate storyPriority storyTasks) includes: each accessor selector]! !

!XPFUserStory methodsFor: 'accessing' stamp: 'mt 2/5/2009 17:48'!
storyActual
	^ storyActual! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:04'!
storyActual: anObject
	storyActual := anObject! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyDescription
	^ storyDescription! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyDescription: anObject
	storyDescription := anObject! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyEstimate
	^ storyEstimate! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyEstimate: anObject
	storyEstimate := anObject! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyName
	^ storyName! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyName: anObject
	storyName := anObject! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyPriority
	^ storyPriority! !

!XPFUserStory methodsFor: 'accessing' stamp: 'SR 1/22/2009 12:05'!
storyPriority: anObject
	storyPriority := anObject! !

Object subclass: #XPFHttpRequestHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Rest'!

XPFHttpRequestHandler subclass: #XPFDefaultHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Rest'!

!XPFDefaultHandler methodsFor: 'request processing' stamp: 'mt 2/2/2009 09:56'!
process: anHttpRequest
	^HttpResponse fromString: 'Default Handler.'! !

!XPFHttpRequestHandler methodsFor: 'request processing' stamp: 'mt 2/2/2009 09:53'!
process: anHttpRequest
	self subclassResponsibility.! !

XPFHttpRequestHandler subclass: #XPFMetaHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Rest'!

!XPFMetaHandler methodsFor: 'request processing' stamp: 'mt 2/2/2009 10:23'!
process: anHttpRequest
	"Takes first element in path as classname and sends message from second path element to that class."

	|pc class selector result|
	pc := (URI fromString: anHttpRequest url) pathComponents.
	class := pc second asSymbol.
	selector := pc third asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HttpResponse fromString: result printString.! !

Object subclass: #XPFLink
	instanceVariableNames: 'endpoints linkname'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Models'!

!XPFLink class methodsFor: 'instance creation' stamp: 'topa 2/9/2009 16:07'!
newFor: someEntities named: aLinkname 
	^ self new
		endpoints: someEntities;
		linkname: aLinkname;
		yourself! !

!XPFLink methodsFor: 'accessing' stamp: 'mig 1/29/2009 18:39'!
endpoints
	^ endpoints! !

!XPFLink methodsFor: 'accessing' stamp: 'mig 1/29/2009 18:39'!
endpoints: anObject
	endpoints := anObject! !

!XPFLink methodsFor: 'accessing' stamp: 'mig 1/29/2009 18:39'!
linkname
	^ linkname! !

!XPFLink methodsFor: 'accessing' stamp: 'mig 1/29/2009 18:39'!
linkname: anObject
	linkname := anObject! !

Object subclass: #XPFLogService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-Rest'!

!XPFLogService class methodsFor: 'service' stamp: 'cs 2/7/2009 14:20'!
log: aCollection
	Transcript show: aCollection first asString; cr.! !

SHTextStylerST80 subclass: #XPFTextStyler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker'!

!XPFTextStyler methodsFor: 'private' stamp: 'mt 1/26/2009 14:55'!
privateFormat: aText
	^super privateFormat: aText.! !

!XPFTextStyler methodsFor: 'private' stamp: 'mt 1/23/2009 15:22'!
privateStyle: aText
	"First, backup current style information about background color in text runs. Then let Shout work. Then restore background color information for text runs."

	| backupRuns |
	backupRuns := aText runs.

	super privateStyle: aText.
	
	backupRuns withStartStopAndValueDo: [:start :stop :attrs | |a|
		a := attrs detect: [:attr | attr isKindOf: XPFTextBackgroundColor] ifNone: [nil].
		a ifNotNil: [aText addAttribute: a from: start to: stop].
	]! !

!XPFTextStyler methodsFor: 'as yet unclassified' stamp: 'mt 1/23/2009 15:24'!
unstyledTextFrom: aText

	| backupRuns result |
	backupRuns := aText runs.

	result := super unstyledTextFrom: aText.
	
	backupRuns withStartStopAndValueDo: [:start :stop :attrs | |a|
		a := attrs detect: [:attr | attr isKindOf: XPFTextBackgroundColor] ifNone: [nil].
		a ifNotNil: [result addAttribute: a from: start to: stop].
	].

	^result.! !

!OBCodeBrowser methodsFor: '*xpf-codemarker-commands' stamp: 'mfp 2/3/2009 16:15'!
cmdMarkAsBad
	^ XPFMarkBadCommand! !

!OBCodeBrowser methodsFor: '*xpf-codemarker-commands' stamp: 'mfp 2/3/2009 16:18'!
cmdMarkAsGood
	^ XPFMarkGoodCommand! !

!OBCodeBrowser methodsFor: '*xpf-codemarker-commands' stamp: 'mfp 2/3/2009 16:23'!
cmdMarkAsUnclear
	^ XPFMarkUnclearCommand! !

!OBCodeBrowser methodsFor: '*xpf-codemarker-commands' stamp: 'mt 2/5/2009 17:45'!
cmdRemoveMarkup
	^ XPFRemoveMarkupCommand! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:08'!
markBadIcon
	^ Icons
			at: #'markBad'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markBadIconContents readStream) ].! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:06'!
markBadIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAACxSURBVDjL7dItC8JQAIXh491H
WbDvD1pEDIIKm1zDxqKi0WIULAZFYc1msF3ErEZlMC9Mhxx/gTAR28KpDxx4QRK/DCXwAbDg
0oLMTShtQF0F5AlwvwHkqy+Zxxs+lwsmvs8DKrIw8DCh8njNLOxRtxrU4yF3MFRhIBFQ2XxG
3W7yXq8xjULGsIsDFwF58zzq0YBpFPDc6XIKp/iFI4S7h5BbWGoFW03gyABVtyzxT8Ab8Xze
i+tkcykAAAAASUVORK5CYII='! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:08'!
markGoodIcon
	^ Icons
			at: #'markGood'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markGoodIconContents readStream) ].! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:06'!
markGoodIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAC5SURBVDjLY/j//z8DJZhh1ADs
BjjsspIC4gb77ZZX7TdbXLVda9Zgs8xEihQDGmZfm/7/0KOD/3ff3/V/6plJ/y3mGjYQbYD9
NsurBx4e+D/10tT/nWc6/i+5sui/+RS9q0QbYLfB/OrWO1v+d5/p+t96qvn/3Auz/pt0aRNv
gPVyk4appyf+X3xl4f/ZF2b+n3Co579+mSrxXrCcZyhlMV2/wbRP56pRq+ZV3SLlBq1EOanR
lEgjAwAXIuIDq5qm/AAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:08'!
markUnknownIcon
	^ Icons
			at: #'markUnknown'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markUnknownIconContents readStream) ].! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:07'!
markUnknownIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAABlSURBVCjPY/jPgB8yDC4FilKK
DfJXZa6KNwhKYVfQkPW/63/b/+T/XA1YFchd7fqf/j/2f+N/5qtYFUhe7fif9D/sf91/BuwK
hBoS/jcBpcP/M2C3gluKrYHhKhA2MEgN2pDEDgEb0f5zlvXgVgAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:08'!
removeMarkupIcon
	^ Icons
			at: #'removeMarkup'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self removeMarkupIconContents readStream) ].! !

!MenuIcons class methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 18:07'!
removeMarkupIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAABbSURBVCjPzdAxDoAgEERRzsFp
95JbGI2ASA2SCOX3Ahtr8tuXTDIO959bCxRfpOitWS5vA+lMJg9JbKCTTmMQ1QS3ThqVQbBB
lsbgpXLYE8lHCXrqLptf9km7Dzv+FwGTaznIAAAAAElFTkSuQmCC'! !

!OBPluggableTextMorphWithShout class methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/26/2009 14:54'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel
	|styler answer stylerClass |
	answer := self new.
	stylerClass := answer stylerClass.
	styler := stylerClass
		ifNotNil:[
			stylerClass new
				view: answer;
				yourself].
	^ answer
		styler: styler;
		on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: getSelectionSel
		menu: getMenuSel! !

!OBPluggableTextMorphWithShout methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/26/2009 14:53'!
stylerClass
	^XPFTextStyler! !

!OBPluggableTextMorphWithShout methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/22/2009 16:59'!
textMorphClass
	"Answer the class used to create the receiver's textMorph"
	^XPFTextMorph! !

OBTextMorphWithShout subclass: #XPFTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker'!

!XPFTextMorph methodsFor: 'as yet unclassified' stamp: 'mt 1/23/2009 11:00'!
editorClass
	"Answer the class used to create the receiver's editor"
	
	^XPFEditor! !

!RunArray class methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/26/2009 09:50'!
scanFrom: strm
	"Read the style section of a fileOut or sources file.  nextChunk has already been done.  We need to return a RunArray of TextAttributes of various kinds.  These are written by the implementors of writeScanOn:"
	| rr vv aa this |
	(strm peekFor: $( ) ifFalse: [^ nil].
	rr := OrderedCollection new.
	[strm skipSeparators.
	 strm peekFor: $)] whileFalse: 
		[rr add: (Number readFrom: strm)].
	vv := OrderedCollection new.	"Value array"
	aa := OrderedCollection new.	"Attributes list"
	[(this := strm next) == nil] whileFalse: [
		this == $, ifTrue: [vv add: aa asArray.  aa := OrderedCollection new].
		this == $a ifTrue: [aa add: 
			(TextAlignment new alignment: (Integer readFrom: strm))].
		this == $f ifTrue: [aa add: 
			(TextFontChange new fontNumber: (Integer readFrom: strm))].
		this == $F ifTrue: [aa add: (TextFontReference toFont: 
			(StrikeFont familyName: (strm upTo: $#) size: (Integer readFrom: strm)))].
		this == $b ifTrue: [aa add: (TextEmphasis bold)].
		this == $i ifTrue: [aa add: (TextEmphasis italic)].
		this == $u ifTrue: [aa add: (TextEmphasis underlined)].
		this == $= ifTrue: [aa add: (TextEmphasis struckOut)].
		this == $n ifTrue: [aa add: (TextEmphasis normal)].
		this == $- ifTrue: [aa add: (TextKern kern: -1)].
		this == $+ ifTrue: [aa add: (TextKern kern: 1)].
		this == $c ifTrue: [aa add: (TextColor scanFrom: strm)]. "color"
		this == $h ifTrue: [aa add: (XPFTextBackgroundColor scanFrom: strm)]. "background color - CodeMarker"
		this == $L ifTrue: [aa add: (TextLink scanFrom: strm)].	"L not look like 1"
		this == $R ifTrue: [aa add: (TextURL scanFrom: strm)].
				"R capitalized so it can follow a number"
		this == $q ifTrue: [aa add: (TextSqkPageLink scanFrom: strm)].
		this == $p ifTrue: [aa add: (TextSqkProjectLink scanFrom: strm)].
		this == $P ifTrue: [aa add: (TextPrintIt scanFrom: strm)].
		this == $d ifTrue: [aa add: (TextDoIt scanFrom: strm)].
		"space, cr do nothing"
		].
	aa size > 0 ifTrue: [vv add: aa asArray].
	^ self runs: rr asArray values: vv asArray
"
RunArray scanFrom: (ReadStream on: '(14 50 312)f1,f1b,f1LInteger +;i')
"! !

!MCStWriter methodsFor: '*XPF' stamp: 'SR 1/29/2009 18:45'!
writeMethodPreamble: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition fullClassName;
		nextPutAll: ' methodsFor: ';
		nextPutAll: definition category asString printString;
		nextPutAll: ' stamp: ';
		nextPutAll: definition timeStamp asString printString;
		nextPutAll: '!!';
		cr! !

!MCStWriter methodsFor: '*XPF' stamp: 'cs 2/5/2009 17:03'!
writeMethodSource: definition
	stream nextChunkPutWithStyle: definition sourceCode.! !

!MCMethodDefinition class methodsFor: '*XPF' stamp: 'SR 1/27/2009 16:38'!
clearCachedDefinitions
	Definitions := nil.
	Instances := nil.! !

!MCMethodDefinition class methodsFor: '*XPF' stamp: 'cs 2/5/2009 16:55'!
forMethodReference: aMethodReference
	| definition |
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference methodSymbol]
		or: [definition className ~= aMethodReference classSymbol]
		or: [definition classIsMeta ~= aMethodReference classIsMeta]
		or: [definition category ~= aMethodReference category])
			ifTrue: [definition := self 
						className: aMethodReference classSymbol
						classIsMeta: aMethodReference classIsMeta
						selector: aMethodReference methodSymbol
						category: aMethodReference category
						timeStamp: aMethodReference timeStamp
						source: aMethodReference sourceCode.
					self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition! !

!MCMethodDefinition methodsFor: '*XPF' stamp: 'cs 2/5/2009 16:57'!
initializeWithClassName: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	className := classString asSymbol.
	selector := selectorString asSymbol.
	category := catString asSymbol.
	timeStamp := timeString.
	classIsMeta := metaBoolean.
	source := sourceString.! !

!MCMethodDefinition methodsFor: '*XPF' stamp: 'cs 2/5/2009 16:59'!
source
	^ source asString withSqueakLineEndings! !

!MCMethodDefinition methodsFor: '*XPF' stamp: 'cs 2/5/2009 16:57'!
sourceCode
	^source asText! !

!MultiNewParagraph methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/22/2009 16:27'!
displayOn: aCanvas using: displayScanner at: somePosition 
	"Send all visible lines to the displayScanner for display"
	| visibleRectangle offset leftInRun line |
	visibleRectangle := aCanvas clipRect.
	offset := somePosition - positionWhenComposed.
	leftInRun := 0.
	(self lineIndexForPoint: visibleRectangle topLeft) 
		to: (self lineIndexForPoint: visibleRectangle bottomRight)
		do: 
			[ :i | 
			line := lines at: i.
			self
				drawBackgroundOn: aCanvas
					at: line;
				displaySelectionInLine: line
					on: aCanvas.
			line first <= line last ifTrue: 
				[ leftInRun := displayScanner 
					displayLine: line
					offset: offset
					leftInRun: leftInRun ] ]! !

!MultiNewParagraph methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/22/2009 16:27'!
drawBackgroundOn: aCanvas at: line 
	| cb1 cb2 textLine bgAttr |
	textLine := self text 
		copyFrom: line first
		to: line last.
	textLine runs withStartStopAndValueDo: 
		[ :start :stop :attributes | 
		bgAttr := attributes 
			detect: [ :each | each isMemberOf: XPFTextBackgroundColor ]
			ifNone: [  ].
		bgAttr ifNotNil: 
			[ cb1 := self characterBlockForIndex: start + (line first - 1).
			cb2 := self characterBlockForIndex: stop + (line first - 1).
			aCanvas 
				fillRectangle: (cb1 left @ line top corner: cb2 right @ line bottom)
				color: bgAttr color ] ]! !

!MultiNewParagraph methodsFor: '*XPF-CodeMarker' stamp: 'mt 2/5/2009 17:42'!
removeBackgroundForSelection
	selectionStart stringIndex == selectionStop ifTrue: [^nil].
	self text removeAttributeOfKind: XPFTextBackgroundColor from: selectionStart stringIndex to: selectionStop stringIndex - 1! !

!MultiNewParagraph methodsFor: '*XPF-CodeMarker' stamp: 'mt 1/22/2009 17:24'!
setBackgroundForSelectionWith: aColor
	selectionStart stringIndex == selectionStop ifTrue: [^nil].
	self text addAttribute: (XPFTextBackgroundColor new color: aColor; yourself) from: selectionStart stringIndex to: selectionStop stringIndex - 1! !

OBCommand subclass: #XPFCodeMarkCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Commands'!

!XPFCodeMarkCommand class methodsFor: 'testing' stamp: 'cs 1/27/2009 17:30'!
takesText
	^ true! !

!XPFCodeMarkCommand methodsFor: 'as yet unclassified' stamp: 'mfp 2/3/2009 16:13'!
execute
	"'inspired' by ORCmdPrettyPrint>>execute"
	| panel morph editor |
	
	panel := requestor browser panels 
		detect: [ :each | each isKindOf: OBDefinitionPanel ]
		ifNone: [ ^ self ].
	morph := OBPluggableTextMorph allSubInstances
		detect: [ :each | each model == panel ]
		ifNone: [ ^ self ].
	editor := morph textMorph editor.
	self mark: target in: editor.
	morph hasUnacceptedEdits: true! !

!XPFCodeMarkCommand methodsFor: 'accessing' stamp: 'cs 1/27/2009 17:31'!
group
	^ #general! !

!XPFCodeMarkCommand methodsFor: 'testing' stamp: 'cs 1/27/2009 17:31'!
isActive
	^ target isKindOf: OBTextSelection! !

!XPFCodeMarkCommand methodsFor: 'as yet unclassified' stamp: 'mfp 2/3/2009 16:14'!
mark: aTarget in: anEditor
	^ self subclassResponsibility! !

XPFCodeMarkCommand subclass: #XPFMarkBadCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Commands'!

!XPFMarkBadCommand methodsFor: 'accessing' stamp: 'mt 2/5/2009 18:09'!
icon
	^#markBadIcon! !

!XPFMarkBadCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:12'!
keystroke
	^ $B! !

!XPFMarkBadCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:11'!
label
	^ 'mark bad'! !

!XPFMarkBadCommand methodsFor: 'editor-bridge' stamp: 'mfp 2/3/2009 16:15'!
mark: aTarget in: anEditor
	anEditor markAsBad: aTarget.! !

XPFCodeMarkCommand subclass: #XPFMarkGoodCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Commands'!

!XPFMarkGoodCommand methodsFor: 'accessing' stamp: 'mt 2/5/2009 18:11'!
icon
	^#markGoodIcon.! !

!XPFMarkGoodCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:20'!
keystroke
	^ $G! !

!XPFMarkGoodCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:20'!
label
	^ 'mark good'! !

!XPFMarkGoodCommand methodsFor: 'editor-bridge' stamp: 'mfp 2/3/2009 16:20'!
mark: aTarget in: anEditor
	anEditor markAsGood: aTarget.! !

XPFCodeMarkCommand subclass: #XPFMarkUnclearCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Commands'!

!XPFMarkUnclearCommand methodsFor: 'accessing' stamp: 'mt 2/5/2009 18:11'!
icon
	^#markUnknownIcon.! !

!XPFMarkUnclearCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:24'!
keystroke
	^ $U! !

!XPFMarkUnclearCommand methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:24'!
label
	^ 'mark unclear'! !

!XPFMarkUnclearCommand methodsFor: 'editor-bridge' stamp: 'mfp 2/3/2009 16:25'!
mark: aTarget in: anEditor
	anEditor markAsUnclear: aTarget.! !

XPFCodeMarkCommand subclass: #XPFRemoveMarkupCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker-Commands'!

!XPFRemoveMarkupCommand methodsFor: 'as yet unclassified' stamp: 'mt 2/5/2009 18:11'!
icon
	^#removeMarkupIcon.! !

!XPFRemoveMarkupCommand methodsFor: 'as yet unclassified' stamp: 'mt 2/5/2009 17:46'!
label
	^'remove markup'! !

!XPFRemoveMarkupCommand methodsFor: 'as yet unclassified' stamp: 'mt 2/5/2009 17:44'!
mark: aTarget in: anEditor
	anEditor removeMarkup: aTarget.! !

TextAttribute subclass: #XPFTextBackgroundColor
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPF-CodeMarker'!

!XPFTextBackgroundColor class methodsFor: 'instance creation' stamp: 'mt 1/26/2009 09:31'!
color: aColor
	^ self new color: aColor! !

!XPFTextBackgroundColor class methodsFor: 'instance creation' stamp: 'mt 1/26/2009 09:42'!
scanFrom: strm
	"read a color in the funny format used by Text styles on files. c125000255 or cblue;"

	| r g b |
	strm peek isDigit
		ifTrue:
			[r := (strm next: 3) asNumber.
			g := (strm next: 3) asNumber.
			b := (strm next: 3) asNumber.
			^ self color: (Color r: r g: g b: b range: 255)].
	"A name of a color"
	^ self color: (Color perform: (strm upTo: $;) asSymbol)! !

!XPFTextBackgroundColor methodsFor: 'comparing' stamp: 'mt 1/22/2009 11:23'!
= other 
	^ (other class == self class) 
		and: [other color = color]! !

!XPFTextBackgroundColor methodsFor: 'html' stamp: 'mt 1/22/2009 11:24'!
closeHtmlOn: aStream 
	"put on the given stream the tag to close the html  
	representation of the receiver"
	aStream nextPutAll: '</span>'! !

!XPFTextBackgroundColor methodsFor: 'accessing' stamp: 'mt 1/22/2009 11:22'!
color
	^ color! !

!XPFTextBackgroundColor methodsFor: 'accessing' stamp: 'mt 1/22/2009 17:51'!
color: aColor
	color := aColor copy alpha: 0.4.! !

!XPFTextBackgroundColor methodsFor: 'scanning' stamp: 'mt 1/22/2009 11:26'!
dominates: other
	^ other class == self class! !

!XPFTextBackgroundColor methodsFor: 'comparing' stamp: 'mt 1/22/2009 11:23'!
hash
	^ color hash! !

!XPFTextBackgroundColor methodsFor: 'html' stamp: 'mt 1/22/2009 11:25'!
openHtmlOn: aStream 
	"put on the given stream the tag to open the html  
	representation of the receiver"
	aStream nextPutAll: '<span style="background-color: #' , color printHtmlString , '">'! !

!XPFTextBackgroundColor methodsFor: 'printing' stamp: 'mt 1/22/2009 11:25'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' code: '; print: color! !

!XPFTextBackgroundColor methodsFor: 'scanning' stamp: 'mt 1/22/2009 11:27'!
writeScanOn: strm
	"Two formats.  c125000255 or cblue;"

	| nn str |
	strm nextPut: $h. "h ... highlight"
	(nn := color name) ifNotNil: [
		(self class respondsTo: nn) ifTrue: [
			^ strm nextPutAll: nn; nextPut: $;]].
	(Array with: color red with: color green with: color blue) do: [:float |
		str := '000', (float * 255) asInteger printString.
		strm nextPutAll: (str copyFrom: str size-2 to: str size)]! !
XPFEntity initialize!
